/* cbs_eval.cpp
 *
 * Mex entry point for the function evaluating a monodimensional b-spline.
 * See cbs_eval.m for documentation.
 *
 * History
 *  2011/06/24: First version
 *
 * Note
 *  CBS is partly based on the original source code of BBS.
 *
 * (c)2011, Florent Brunet.
 */

/* This file is part of CBS.
 *
 * CBS is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CBS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CBS.  If not, see <http://www.gnu.org/licenses/>.
 */
 
#include "cbs_mex.h"
#include "cbs.h"
#include <string.h>

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
    cbs_t cbs;
    double *t = NULL;
    int nval_t;
    int dt = 0;
    double *ctrlpts = NULL;
    double *val = NULL;

    
    // INPUT ARGUMENTS
    if (nrhs != 3 && nrhs != 4) 
        mexErrMsgTxt("Three or four input arguments required.");
    
    // No need to check "prhs[0]" for validity since it is done in "array_to_cbs"
    array_to_cbs(prhs[0], &cbs);
    
    // prhs[1]: control points
    check_real(prhs[1], "ctrlpts");
    check_cbs_ctrlpts_size(prhs[1], &cbs);
    ctrlpts = mxGetPr(prhs[1]);
    
    // prhs[2] : t
    check_real(prhs[2], "t");
    nval_t = mxGetNumberOfElements(prhs[2]);

    t = mxGetPr(prhs[2]);
    
    mwSize ndims_t = mxGetNumberOfDimensions(prhs[2]);
    const mwSize* dims_t = mxGetDimensions(prhs[2]);
    
    // prhs[3]: optional order of derivation
    if (nrhs == 4) {
        check_real_scalar(prhs[3], "dt");
        dt = (int)mxGetScalar(prhs[3]);
    }
    
    // OUTPUT ARGUMENTS
    // plhs[0] : val
    if (nlhs == 0) 
        mexErrMsgTxt("At least one output argument is required.");
    else if (nlhs == -1) {
        plhs[0] = mxCreateDoubleMatrix(cbs.valdim, nval_t, mxREAL);
        val = mxGetPr(plhs[0]);
        // ACTUAL EVALUATION
        eval(&cbs, ctrlpts, t, nval_t, val, dt);
    } else {
        if (nlhs != cbs.valdim)
            mexErrMsgTxt("The number of output arguments must be equal to 1 or to the dimension of the B-spline.");
        
        // ACTUAL EVALUATION
        double *val = (double*)mxMalloc(cbs.valdim*nval_t*sizeof(double));
        eval(&cbs, ctrlpts, t, nval_t, val, dt);
        
        // Allocation of the output arguments
        double **val_cur_dim = (double**)mxMalloc(cbs.valdim*sizeof(double));
        for (int i = 0 ; i < cbs.valdim ; ++i) {
            plhs[i] = mxCreateNumericArray(ndims_t, dims_t, mxDOUBLE_CLASS, mxREAL);
            val_cur_dim[i] = mxGetPr(plhs[i]);
        }
        
        // Copy the result into the output arguments
        int k = 0;
        for (int j = 0 ; j < nval_t ; ++j)
            for (int i = 0; i < cbs.valdim; ++i) {
                val_cur_dim[i][j] = val[k];
                ++k;
            }
        mxFree(val);
        mxFree(val_cur_dim);
    }
}
